@@ -1,28 +1,51 @@
-## hw.test.R (2012-11-20)
+## hw.test.R (2014-07-10)
 
 ##   Test of Hardy--Weinberg Equilibrium
 
-## Copyright 2009-2012 Emmanuel Paradis
+## Copyright 2009-2014 Emmanuel Paradis
 
 ## This file is part of the R-package `pegas'.
-## See the file ../COPYING for licensing issues.
+## See the file ../DESCRIPTION for licensing issues.
+
+## This function outputs the multinomial coefficients with input:
+## x: output from expand.genotype(, matrix = TRUE), see code in simulate.genotype()
+.multinomialCoef <- function(x)
+{
+    n <- nrow(x)
+    numerator <- factorial(ncol(x))
+    coef <- numeric(n)
+    for (i in seq_len(n))
+        coef[i] <- numerator/prod(factorial(tabulate(x[i, ])))
+    coef
+}
+
+proba.genotype <- function(alleles = c("1", "2"), p, ploidy = 2)
+{
+    o <- .sort.alleles(alleles, index.only = TRUE)
+    alleles <- alleles[o]
+    p <- if (missing(p)) rep(1/length(alleles), length(alleles)) else p[o]
+    geno <- expand.genotype(length(alleles), ploidy = ploidy, matrix = TRUE)
+    P <- .multinomialCoef(geno) * apply(geno, 1, function(i) prod(p[i]))
+    names(P) <- apply(matrix(alleles[geno], ncol = ploidy), 1, paste, collapse = "/")
+    P
+}
 
 expand.genotype <- function(n, alleles = NULL, ploidy = 2, matrix = FALSE)
 {
-    if (!is.null(alleles)) n <- length(alleles)
-    if (ploidy == 2) {
-        unan <- 1:n
-        ans <- matrix(c(unan, unan), n, 2)
-        ans <- rbind(ans, t(combn(unan, 2)))
+    if (!is.null(alleles)) {
+        alleles <- .sort.alleles(alleles)
+        n <- length(alleles)
     }
-    if (ploidy == 4) {
-        ans <- matrix(0, 0, 4)
-        for (a1 in 1:n)
-            for (a2 in a1:n)
-                for (a3 in a2:n)
-                    for (a4 in a3:n)
-                        ans <- rbind(ans, c(a1, a2, a3, a4))
+    ans <- matrix(0L, 0L, ploidy)
+    foo <- function(i, a) {
+        for (x in a:n) {
+            g[i] <<- x
+            if (i < ploidy) foo(i + 1L, x) else ans <<- rbind(ans, g)
+        }
     }
+    g <- integer(ploidy)
+    foo(1L, 1L)
+    dimnames(ans) <- NULL
     if (is.character(alleles))
         ans <- matrix(alleles[ans], ncol = ploidy)
     if (!matrix)
@@ -32,45 +55,11 @@ expand.genotype <- function(n, alleles = NULL, ploidy = 2, matrix = FALSE)
 
 hw.test <- function(x, B = 1000)
 {
-    multinomialCoef <- function(x) {
-        n <- nrow(x)
-        numerator <- factorial(ncol(x))
-        coef <- numeric(n)
-        for (i in 1:n) {
-            y <- tabulate(x[i, ])
-            y <- y[y > 0]
-            coef[i] <- numerator/prod(factorial(y))
-        }
-        coef
-    }
     test.polyploid <- function(x, ploidy) {
-        nms <- names(x$allele)
-        Nall <- length(nms)
-        all.prop <- x$allele/sum(x$allele)
-        all.geno <- expand.genotype(Nall, ploidy = ploidy, matrix = TRUE)
-        Ngeno <- nrow(all.geno)
-        ## expected proportions:
-        E <- numeric(Ngeno)
-        for (i in 1:Ngeno) E[i] <- prod(all.prop[all.geno[i, ]])
-        E <- multinomialCoef(all.geno) * E
-        names(E) <- expand.genotype(alleles = nms, ploidy = ploidy)
-        O <- E
-        O[] <- 0
-        O[names(x$genotype)] <- x$genotype
-        E <- sum(O) * E
-        chi2 <- sum((O - E)^2/E)
-        DF <- length(E) - Nall
-        c(chi2, DF, 1 - pchisq(chi2, DF))
-    }
-    test.diploid <- function(x) {
+        if (ploidy < 2) return(rep(NA_real_, 3))
         nms <- names(x$allele)
         all.prop <- x$allele/sum(x$allele)
-        E <- all.prop %o% all.prop
-        ## expected proportions (homozygotes first):
-        E <- c(diag(E), 2 * E[col(E) < row(E)])
-        tmp <- combn(nms, 2)
-        names(E) <- c(paste(nms, nms, sep="/"),
-                      paste(tmp[1, ], tmp[2, ], sep="/"))
+        E <- proba.genotype(nms, all.prop, ploidy = ploidy)
         O <- E
         O[] <- 0
         O[names(x$genotype)] <- x$genotype
@@ -80,22 +69,17 @@ hw.test <- function(x, B = 1000)
         c(chi2, DF, 1 - pchisq(chi2, DF))
     }
     y <- summary.loci(x)
-    plo <- getPloidy(x)[1]
-    if (plo == 2) {
-        ans <- lapply(y, test.diploid)
-    } else {
-        ans <- lapply(y, test.polyploid, ploidy = plo)
-        if (B) {
-            warning("no Monte Carlo test available for polyploids")
-            B <- 0
-        }
-    }
-    ans <- matrix(unlist(ans), ncol = 3, byrow = TRUE)
+    ploidy <- getPloidy(x)
+    ans <- t(mapply(test.polyploid, y, ploidy = ploidy))
     dimnames(ans) <- list(names(y), c("chi^2", "df", "Pr(chi^2 >)"))
     if (B) {
         LN2 <- log(2)
-        test.mc <- function(x) {
+        test.mc <- function(x, ploidy) {
             ## accept only diploids for the moment
+            if (ploidy != 2) {
+                warning("no Monte Carlo test available for polyploids")
+                return(NA_real_)
+            }
             n <- sum(x$genotype) # Nb of individuals
             Nall <- length(x$allele) # Nb of alleles
             all.geno <- expand.genotype(Nall)
@@ -104,6 +88,7 @@ hw.test <- function(x, B = 1000)
             ## the constant term below is dropped:
             ## lfactorial(n) - lfactorial(2*n) + sum(lfactorial(x$allele))
             ma <- unlist(mapply(rep, 1:Nall, x$allele))
+            homoz.nms <- paste(1:Nall, 1:Nall, sep = "/") # to help find the homozygotes (see below)
             for (k in 1:B) {
                 m <- sample(ma)
                 dim(m) <- c(n, 2)
@@ -112,11 +97,8 @@ hw.test <- function(x, B = 1000)
                 m[o, 1:2] <- m[o, 2:1]
                 s <- factor(paste(m[, 1], m[, 2], sep = "/"), levels = all.geno)
                 f <- tabulate(s, Ngeno)
-                names(f) <- all.geno
-                ## we know the homozygotes are the 1st Nall genotypes:
-                #h <- unlist(lapply(strsplit(names(f), "/"),
-                #                   function(x) length(unique(x)))) == 1
-                p[k] <- -sum(lfactorial(f)) + LN2*sum(f[-(1:Nall)])
+                ## find the homozygotes:
+                p[k] <- -sum(lfactorial(f)) + LN2 * sum(f[-match(homoz.nms, all.geno)])
             }
             ## find the homozygotes:
             h <- unlist(lapply(strsplit(names(x$genotype), "/"),
@@ -124,7 +106,7 @@ hw.test <- function(x, B = 1000)
             p0 <- -sum(lfactorial(x$genotype)) + LN2*sum(x$genotype[!h])
             sum(p <= p0)/B
         }
-        ans <- cbind(ans, "Pr.exact" = unlist(lapply(y, test.mc)))
+        ans <- cbind(ans, "Pr.exact" = mapply(test.mc, y, ploidy))
     }
     ans
 }
